//
//  PCQRCode.h
//  PayControl
//
//  Created by Roman Kolesnichenko on 13.08.16.
//  Copyright © 2016 SafeTech. All rights reserved.
//

#import "PCKey.h"

/** Тип  QR-кода. */
typedef NS_ENUM(NSUInteger, PCQRCodeType)
{
    PCQRCodeTypeFullKey = 201,
    PCQRCodeTypeHalfKey = 202,
    PCQRCodeTypeTransaction = 203
};

typedef enum : NSUInteger {
    PCKeyPasswordPolicyMinAllowed = 0,
    PCKeyPasswordPolicyMin = 1,
    PCKeyPasswordPolicyMaxSymbols = 2,
    PCKeyPasswordPolicyMaxSymbolsAndDigits = 3,
} PCKeyPasswordPolicy;

/** Базовый класс парсинга данных из строки QR-кода. Должен использоваться через метод класса qrCodeWithString: */
@interface PCQRCode : NSObject

/// ****************************************************************************
/// @name Методы класса
/// ****************************************************************************

/** 
 Создает экземпляр PCKeyQRCode или PCTransactionQRCode
 @param qrCodeString строка с данными QR-кода ключа или транзакции
 @return Для валидной строки из QR-кода возвращает экземпляр класса PCKeyQRCode или PCTransactionQRCode иначе nil */
+ (nullable id)qrCodeWithString:(NSString*)qrCodeString;

/// ****************************************************************************
/// @name Свойства
/// ****************************************************************************
/** Тип  QR-кода. см. PCQRCodeType */
@property (nonatomic, readonly) PCQRCodeType codeType;

@end

/** Объект PCKeyQRCode реализующий доступ к значениям свойств из валидных данных QR-кода ключа */
@interface PCKeyQRCode : PCQRCode

/// ****************************************************************************
/// @name Методы класса
/// ****************************************************************************

/** Создание экзепляра из строки с валидными данными QR-кода.
    @param qrCodeString строка с данными QR-кода
    @return экземпляр PCKeyQRCode или nil если данные переданные в параметры qrCodeString не являются валидными для QR-кода ключа.
*/
+ (nullable instancetype)keyQRCodeWithString:(NSString*)qrCodeString;

/// ****************************************************************************
/// @name Свойства
/// ****************************************************************************

/** Идентификатор пользователя PayControl */
@property(nonatomic, readonly, nullable) NSString *userID;

/** Зарегистрированный MFMSecurityToken */
@property(nonatomic, readonly, nullable) NSString *registeredMFMSecurityToken;

/** Значение ключа */
@property(nonatomic, readonly, nullable) NSString *keyValue;

/** Дата истечения срока действия ключа в виде строки "dd.MM.yyyy" */
@property(nonatomic, readonly, nullable) NSString *expirationDateString;

/** Дата истечения срока действия ключа */
@property(nonatomic, readonly, nullable) NSDate *expirationDate;

/** Тип ключа: DSS или Regular */
@property(nonatomic, readonly) PCKeyType keyType;

/** Флаги ключа */
@property(nonatomic) int keyFlags;

/** Идентификатор прикладной системы */
@property(nonatomic, readonly, nullable) NSString *systemID;

/** Строка с URL для подтверждения Online */
@property(nonatomic, readonly, nullable) NSString *onlineConfirmURL;

/** Строка с URL PС Interaction Server */
@property(nonatomic, readonly, nullable) NSString *interactionExternalURL;

@end

/** Объект PCTransactionQRCode реализующий доступ к значениям свойств из валидных данных QR-кода транзакции */
@interface PCTransactionQRCode : PCQRCode

/// ****************************************************************************
/// @name Методы класса
/// ****************************************************************************

/** Создание экзепляра из строки с валидными данными QR-кода.
 @param qrCodeString строка с данными QR-кода
 @return экземпляр PCTransactionQRCode или nil если данные переданные в параметры qrCodeString не являются валидными для QR-кода транзакции.
 */
+ (nullable instancetype)transactionQRCodeWithString:(NSString*)qrCodeString;

/// ****************************************************************************
/// @name Свойства
/// ****************************************************************************

/** Идентификатор прикладной системы */
@property(nonatomic, readonly, nullable) NSString *systemID;

/** Идентификатор пользователя PayControl */
@property(nonatomic, readonly, nullable) NSString *userID;

/** Идентификатор транзакции */
@property(nonatomic, readonly, nullable) NSString *transactionID;

/** Предпочитаемый тип подтверждения */
@property(nonatomic, readonly) PCConfirmationType confirmationType;

/** Длина кода подтверждения */
@property(nonatomic, readonly) NSUInteger codeLength;

/** Данные транзакции */
@property(nonatomic, readonly, nullable) NSString *data;


@end
